'''
Plays audible reminders on a set interval for tasks in a text area formatted like "HH:MMam arbitrary text".

@author: Peter Parente
@organization: IBM Corporation
@copyright: Copyright (c) 2006 IBM Corporation
@license: Common Public License 1.0

All rights reserved. This program and the accompanying materials are made
available under the terms of the Common Public License v1.0 which accompanies
this distribution, and is available at
U{http://www.opensource.org/licenses/cpl1.0.php}
'''
# import useful modules for Perks
import Perk, Task, LSRConstants
from POR import POR
from i18n import bind, _
import datetime as dt
from datetime import datetime

__uie__ = dict(kind='perk', tier=None, all_tiers=False)

class ReminderState(Perk.PerkState):
  '''
  Defines user configurable settings including auditory icons, intervals
  between checks, and announcement thresholds.
  '''
  SoonSound = '14642_psakion_chimes06_01_20b_dpa_crop1.wav'
  DueSound = '19026_han1_claxon.wav'
  SoonMinutes = 20
  DueMinutes = 5
  CheckMinutes = 5
  
  def getSettings(self):
    '''
    Gets groups of user settings for configuration.
    '''
    g = self._newGroup()
    g1 = g.newGroup('Sounds')
    # files are relative to the location of this module
    g1.newFilename('SoonSound', 'Upcoming task sound', __file__)
    g1.newFilename('DueSound', 'Task due sound', __file__)
    g2 = g.newGroup('Times')
    g2.newNumeric('SoonMinutes', 'Upcoming reminder', 1, 180, 0)
    g2.newNumeric('DueMinutes', 'Due reminder', 1, 180, 0)
    g2.newNumeric('CheckMinutes', 'Minutes between reminders', 1, 60, 0)
    return g

class ReminderPerk(Perk.Perk):
  '''
  Audibly reminds users of 
  '''
  STATE = ReminderState
  
  def init(self):
    '''
    Registers a task to run on a one minute timed interval. That task will
    obey the user setting for intervals between announcements.
    '''
    self.registerTask(AnnounceTasks(60, 'announce tasks'))
    self.doTask('announce tasks', first=True)

  def getDescription(self):
    return _('Provides audible reminders for tasks. Load on a window having '
             'an editable text area.')

class AnnounceTasks(Task.TimerTask):
  '''
  Announces tasks on a set interval.
  '''
  def init(self):
    '''
    Initialize instance variables.
    '''
    self.text_por = None
    self.count = 0
  
  def execute(self, layer, first=False, **kwargs):
    '''
    Check if it's time to do another announcement. If not or the schedule 
    goes missing, just return.
    '''
    if self.count < self.perk_state.CheckMinutes:
      # quit now if it's not time to make another announcement
      self.count += 1
      return
    else:
      self.count = 0
    # stop all output
    self.stopAll()
    # get the schedule
    text = self.getScheduleText()
    if text is None: return
    now = datetime.now()
    if first:
      # give an intro message
      appts = self.parseText(text, now.date())
      self.sayItem(text=(self.perk_state.CheckMinutes, len(appts)),
                   template='reminders started. notifying every %d minutes. '
                   '%d tasks scheduled for today')
    else:
      # do a notification
      appts = self.parseText(text, now.date())
      play = self.notifyDue(now, appts)
      self.notifySoon(now, appts, play)

  def notifyDue(self, now, appts):
    '''
    Play a sound and speak tasks about to become due or overdue.
    '''
    due = []
    play = False
    # loop through appointments and announce those that are overdue
    for time, desc in appts:
      diff = time-now
      minutes = diff.seconds/60.0
      template = None
      if diff.days < 0:
        template = '%s is overdue'
      elif minutes <= self.perk_state.DueMinutes:
        template = '%s now'
      if template:
        if not play:
          self.play(self.perk_state.DueSound)
          play = True
        self.sayItem(text=desc, template=template)
    return play
  
  def notifySoon(self, now, appts, play):
    '''
    Plays a sound and speak tasks coming due soon.
    '''
    # loop through appointments and announce those coming up soon
    for time, desc in appts:
      diff = time-now
      minutes = diff.seconds/60.0
      if (minutes <= self.perk_state.SoonMinutes and 
          minutes > self.perk_state.DueMinutes):
        if not play:
          self.play(self.perk_state.SoonSound)
          play = True
        self.sayItem(text=(minutes, desc), template='in %d minutes, %s')
    return play

  def parseText(self, text, date):
    '''
    Parses the text to find potential appointments. Ignores lines that are
    not valid appts.
    '''
    appts = []
    for line in text.split('\n'):
      # ignore whitespace
      line = line.strip()
      if not line: continue
      off = line.find(' ')
      time = line[:off]
      desc = line[off:]
      try:
        # get time
        hour, min = time.split(':')
      except ValueError:
        # missing minutes are OK
        hour, ap = time[:-2], time[-2:]
        min = '00'
      else:
        min, ap = min[:-2], min[-2:]
      try:
        # convert am/pm to 24hr clock
        hour = int(hour)
        if ap.lower() == 'pm' and hour < 12:
          hour += 12
        # build a datetime object
        t = datetime.combine(date, dt.time(hour,int(min)))
      except Exception:
        continue
      appts.append((t, desc.strip()))
    return appts

  def getScheduleText(self):
    '''
    Locates the text area containing the schedule.
    '''
    if self.text_por is not None:
      # reuse the previously discovered text area
      return self.getAccText(self.text_por)
    else:
      # first run, find a text area and latch on
      root = self.getRootAcc()
      self.text_por = self.findAccByRole('text', root)
      return self.getAccText(self.text_por)
